#include <iostream>
#include <map>

void print_map(const auto& m) {
	for (auto& [k, v]: m) {
		std::cout << k << ": " << v << '\n';
	}
	std::cout << "---\n";
}

int main() {
	std::map<int, char> m = { { 42, 'a' }, { 17, 'x' }, { 0, 'a' } };
	print_map(m);

	m.insert({ 0, 'c' });
	m.emplace(11, 'd');
	print_map(m);

	m.insert_or_assign(0, 'c');
	print_map(m);

	m[10] = 'w';
	m[42] = 'q';
	print_map(m);

	if (auto [it, success] = m.emplace(17, 'z'); success) {
		std::cout << "Successfully inserted.\n";
	} else {
		std::cout << "Key already present with value: "
		          << it->second << '\n';
	}

	std::map<char, std::tuple<int, int>> mt;

	mt.insert({ 'a', { 1, 2 } });
	mt.emplace('b', std::tuple{ 3, 4 });  // the type needs to be here :-/
	mt.try_emplace('c', 7, 8);

	for (auto& [k, v] : mt) {
		// k = 'x'; // error
		std::get<0>(v) = 0;
	}

	for (const auto& [k, v] : mt) {
		auto [x, y] = v;
		std::cout << k << ": (" << x << ", " << y << ")\n";
	}
	std::cout << "---\n";

	std::map<std::tuple<int, int>, std::tuple<int, int>> wormhole;
	wormhole[{ 0, 0 }] = { 1234, 5678 };

	// wrong!
	if (wormhole[{ 0, 7 }] == std::tuple{ 42, 17 }) {
		std::cout << "Yes\n";
	}

	// right
	if (auto it = wormhole.find({ 0, 9 });
			it != wormhole.end() &&
			it->second == std::tuple{ 42, 17 }) {
		std::cout << "Yes\n";
	}

	for (const auto& [from, to] : wormhole) {
		auto& [fx, fy] = from;
		auto& [tx, ty] = to;
		std::cout << "(" << fx << ", " << fy << ") -> ("
		          << tx << ", " << ty << ")\n";
	}
	std::cout << "---\n";
}
