#include <iostream>
#include <set>
#include <tuple>

void print_set(const auto& vec) {
	std::cout << "set of size " << vec.size() << ":";
	for (auto& elem : vec) {
		std::cout << ' ' << elem;
	}
	std::cout << '\n';
}

int main() {
	std::set<int> empty;
	print_set(empty);

	std::set s = { 7, 2, 5, 2, 5 };
	print_set(s);

	s.insert(0);
	print_set(s);

	if (auto [it, inserted] = s.insert(5); inserted) {
		std::cout << "inserted new value: " << *it << '\n';
	} else {
		std::cout << "value already there: " << *it << '\n';
	}

	print_set(s);

	s.erase(2);
	print_set(s);

	std::cout << (s.contains(0) ? "yes" : "no") << '\n';

	auto it = s.find(7);
	if (it != s.end()) {
		std::cout << "found element " << *it << ", removing it\n";
		s.erase(it);
	}

	print_set(s);

	std::set<std::tuple<int, double>> st = { { 1, 3.14 }, { 2, 7.2 } };
	st.insert(std::tuple{ 1, 9.19 });
	// better:
	st.insert({ 2, 0.01 });
	// even better:
	st.emplace(1, 3.14);

	for (auto [i, d] : st) {
		std::cout << " (" << i << ", " << d << ")";
	}
	std::cout << '\n';
}
